ParseDecimalError

Enum ParseDecimalError 

pub enum ParseDecimalError {
    SyntaxError,
    OutOfRange,
}
Error that can be returned when parsing Fixed or DynamicDecimal.

Variants



SyntaxError

Invalid syntax.



OutOfRange

Out of valid range.

Underflow is rounded to zero, so this error is only returned when the absolute value exceeds the type’s range.