DynamicDecimal

Struct DynamicDecimal 

pub struct DynamicDecimal { /* private fields */ }
Decimal real number with 38 digits of precision and dynamic scale.

This type is primarily meant as a serialized form of Fixed that does not require parameterization. Any Fixed can be converted into DynamicDecimal and then converted back into any other Fixed, possibly with a different precision and scale, without loss of precision (beyond that inherent in change of scale, if any).

Representation

The number is represented as an i128 significand and a u8 scale, that together express the value significand * 10**-scale.

Invariants

These invariants on significand and scale ensure that the value is in canonical form:

  • If significand != 0 && scale != 0, then significand must not be a multiple of 10.

  • If significand != 0 && scale == 0, then significand may be a multiple of 10. (This is a “denormalized” representation with fewer digits of precision than other forms.)

  • If significand == 0, then scale must be zero.

Arithmetic

There are various arithmetic traits implemented for DynamicDecimal, but they are intended for limited uses; in particular, these functions will panic for overflows.

Implementations

Source

impl DynamicDecimal

pub const MAX: Self

The largest DynamicDecimal value (i128::MAX).

pub const MIN: Self

The smallest DynamicDecimal value (i128::MIN).

pub const ZERO: Self

0 as DynamicDecimal.

pub const ONE: Self

1 as DynamicDecimal.

pub const fn new(sig: i128, exponent: u8) -> Self

Constructs a new DynamicDecimal with value sig * 10**-exponent.

pub const fn significand(&self) -> i128

Returns the canonical significand.

pub const fn exponent(&self) -> u8

Returns the canonical scale.

pub fn trunc(&self) -> Self

Truncates the value to an integer by discarding digits after decimal point

pub fn trunc_digits(&self, digits: u8) -> Self

Returns this value, truncated toward zero at digits after the decimal point.

pub fn floor(&self) -> Self

Floor of the value

pub const fn is_negative(self) -> bool

True if value is negative

pub const fn abs(self) -> Self

The absolute value